import xarray
from dateutil.parser import parse
from datetime import datetime, timedelta
import typing
import numpy as np

from .base import IRI

__all__ = ["datetimerange", "timeprofile", "geoprofile"]


def datetimerange(start: datetime, end: datetime, step: timedelta) -> typing.List[datetime]:
    """like range() for datetime"""
    if isinstance(start, str):
        start = parse(start)

    if isinstance(end, str):
        end = parse(end)

    assert isinstance(start, datetime)
    assert isinstance(end, datetime)
    assert isinstance(step, timedelta)

    return [start + i * step for i in range((end - start) // step)]


def timeprofile(tlim: tuple, dt: timedelta, altkmrange: list, glat: float, glon: float) -> xarray.Dataset:
    """compute IRI altitude profile over time range for fixed lat/lon
    """

    times = datetimerange(tlim[0], tlim[1], dt)

    iono: xarray.Dataset = None

    f107 = []
    ap = []
    for time in times:
        iri = IRI(time, altkmrange, glat, glon)
        if iono is None:
            iono = iri
        else:
            iono = xarray.concat((iono, iri), dim="time")

        f107.append(iri.f107)
        ap.append(iri.ap)

    iono.attrs = iri.attrs
    iono.attrs["f107"] = f107
    iono.attrs["ap"] = ap

    return iono


def geoprofile(latrange: typing.Sequence[float], glon: float, altkm: float, time: datetime) -> xarray.Dataset:
    """compute IRI altitude profiles at time, over lat or lon range
    """

    glat = np.arange(*latrange)

    iono: xarray.Dataset = None

    f107 = []
    ap = []
    for l in glat:
        iri = IRI(time, altkmrange=[altkm] * 3, glat=l, glon=glon)
        if iono is None:
            iono = iri
        else:
            iono = xarray.concat((iono, iri), dim="glat")

        f107.append(iri.f107)
        ap.append(iri.ap)

    iono.attrs = iri.attrs
    iono.attrs["f107"] = f107
    iono.attrs["ap"] = ap

    return iono



"""
A generic, clean way to build C/C++/Fortran code "build on run"

Michael Hirsch, Ph.D.
https://www.scivision.dev
"""
import shutil
from pathlib import Path
import subprocess
import typing
import sys
import pkg_resources
import argparse
import logging

R = Path(__file__).resolve().parent
SRCDIR = R
BINDIR = SRCDIR / "build"


def build(src_dir: Path = SRCDIR, bin_dir: Path = BINDIR, build_sys: str = "cmake"):
    """
    attempts build with Meson or CMake
    """
    if build_sys == "meson":
        meson_setup(src_dir, bin_dir)
    elif build_sys == "cmake":
        cmake_setup(src_dir, bin_dir)
    else:
        raise ValueError("Unknown build system {}".format(build_sys))


def cmake_setup(src_dir: Path, bin_dir: Path):
    """
    attempt to build using CMake >= 3
    """

    cmake_exe = check_cmake_version("3.13")

    cfgfn = bin_dir / "CMakeCache.txt"
    if cfgfn.is_file():
        cfgfn.unlink()

    subprocess.run([cmake_exe, "-S", str(src_dir), "-B", str(bin_dir)])

    subprocess.run([cmake_exe, "--build", str(bin_dir), "--parallel"])


def meson_setup(src_dir: Path, bin_dir: Path):
    """
    attempt to build with Meson + Ninja
    """
    args: typing.List[str] = []
    meson_exe = shutil.which("meson")

    if not meson_exe:
        raise FileNotFoundError("Meson not available")

    if (bin_dir / "build.ninja").is_file():
        args += ["--wipe"]

    cmd = [meson_exe, "setup", str(bin_dir), str(src_dir)] + args
    logging.debug(cmd)
    subprocess.run(cmd)

    subprocess.run([meson_exe, "test", "-C", str(bin_dir)])


def check_cmake_version(min_version: str) -> str:
    cmake = shutil.which("cmake")
    if not cmake:
        raise FileNotFoundError("CMake not found")

    cmake_version = subprocess.check_output([cmake, "--version"], universal_newlines=True).split()[2]

    pmin = pkg_resources.parse_version(min_version)
    pcmake = pkg_resources.parse_version(cmake_version)

    if pcmake < pmin:
        raise ValueError(f"CMake {cmake_version} < {min_version}")

    return cmake


def get_libpath(bin_dir: Path, stem: str) -> Path:
    if sys.platform in ("win32", "cygwin"):
        dllfn = bin_dir / ("lib" + stem + ".dll")
    elif sys.platform == "linux":
        dllfn = bin_dir / ("lib" + stem + ".so")
    elif sys.platform == "darwin":
        dllfn = bin_dir / ("lib" + stem + ".dylib")
    else:
        raise ValueError(f"Unknown platform {sys.platform}")

    if not dllfn.is_file():
        dllfn = None

    return dllfn


if __name__ == "__main__":
    p = argparse.ArgumentParser()
    p.add_argument("buildsys", choices=["cmake", "meson"])
    P = p.parse_args()

    build(SRCDIR, BINDIR, P.buildsys)



import subprocess
from dateutil.parser import parse
from datetime import datetime
from pathlib import Path
import xarray
import io
import shutil
import numpy as np
import typing

from .build import build

R = Path(__file__).resolve().parent
datadir = R / "data"
SDIR = R
BDIR = SDIR / "build"
exe_name = "iri2016_driver"

EXE = shutil.which(exe_name, path=str(BDIR))
if EXE is None:
    build(SDIR, BDIR)

    EXE = shutil.which(exe_name, path=str(BDIR))
    if EXE is None:
        raise ModuleNotFoundError(f"could not build {exe_name}, binary not found in {BDIR}")

SIMOUT = ["ne", "Tn", "Ti", "Te", "nO+", "nH+", "nHe+", "nO2+", "nNO+", "nCI", "nN+"]

__all__ = ["IRI"]


def IRI(time: datetime, altkmrange: typing.Sequence[float], glat: float, glon: float) -> xarray.Dataset:

    if isinstance(time, str):
        time = parse(time)

    assert len(altkmrange) == 3, "altitude (km) min, max, step"
    assert isinstance(glat, float) and isinstance(glon, float), "glat, glon is scalar"

    cmd = [
        str(EXE),
        str(time.year),
        str(time.month),
        str(time.day),
        str(time.hour),
        str(time.minute),
        str(time.second),
        str(glat),
        str(glon),
        str(altkmrange[0]),
        str(altkmrange[1]),
        str(altkmrange[2]),
        str(datadir),
    ]

    ret = subprocess.check_output(cmd, universal_newlines=True, cwd=str(BDIR))  # str for Windows
    # %% get altitude profile data
    Nalt = int((altkmrange[1] - altkmrange[0]) // altkmrange[2]) + 1

    arr = np.genfromtxt(io.StringIO(ret), max_rows=Nalt)
    arr = np.atleast_2d(arr)
    assert arr.ndim == 2 and arr.shape[1] == 12, "bad text data output format"

    dsf = {k: (("alt_km"), v) for (k, v) in zip(SIMOUT, arr[:, 1:].T)}
    altkm = arr[:, 0]
    # %% get parameter data
    arr = np.genfromtxt(io.StringIO(ret), skip_header=Nalt)
    assert arr.ndim == 1 and arr.size == 100, "bad text data output format"
    # %% assemble output
    iono = xarray.Dataset(
        dsf, coords={"time": [time], "alt_km": altkm, "glat": glat, "glon": glon}, attrs={"f107": arr[40], "ap": arr[51]}
    )

    for i, p in enumerate(["NmF2", "hmF2", "NmF1", "hmF1", "NmE", "hmE"]):
        iono[p] = (("time"), [arr[i]])

    iono["TEC"] = (("time"), [arr[36]])
    iono["EqVertIonDrift"] = (("time"), [arr[43]])
    iono["foF2"] = (("time"), [arr[99]])

    return iono



#!/usr/bin/env python
import setuptools  # noqa: F401
from pathlib import Path

"""
Because of bad bugs in IRI2016 itself, present even in plain Fortran usage, we can't safely use F2py, bad data can result.

src = [  # 'iriwebg.for',
    'irisub.for', 'irifun.for',
    'iritec.for', 'iridreg.for', 'igrf.for', 'cira.for', 'iriflip.for']

F = Path('src')
src = [str(F/s) for s in src]

ext = Extension(name='iri16', sources=src,
                f2py_options=['only:', 'iri_sub', ':'],  # ['skip:', 'dfridr', ':'],
                extra_f77_compile_args=['-w'])
"""

R = Path("iri2016") / "data"
iridata = list(
    map(
        str,
        (
            list((R / "ccir").glob("*.asc"))
            + list((R / "igrf").glob("*.dat"))
            + list((R / "index").glob("*.dat"))
            + list((R / "mcsat").glob("*.dat"))
            + list((R / "ursi").glob("*.asc"))
        ),
    )
)

iridata = list(map(str, iridata))  # even for Numpy 1.14 due to numpy.distutils

setuptools.setup(
    # ext_modules=[ext],
    data_files=iridata
)



import xarray
from matplotlib.pyplot import figure


def timeprofile(iono: xarray.Dataset):

    fig = figure(figsize=(16, 12))
    axs = fig.subplots(3, 1, sharex=True).ravel()

    fig.suptitle(
        f"{str(iono.time[0].values)[:-13]} to "
        f"{str(iono.time[-1].values)[:-13]}\n"
        f"Glat, Glon: {iono.glat.item()}, {iono.glon.item()}"
    )

    ax = axs[0]
    ax.plot(iono.time, iono["NmF2"], label="N$_m$F$_2$")
    ax.plot(iono.time, iono["NmF1"], label="N$_m$F$_1$")
    ax.plot(iono.time, iono["NmE"], label="N$_m$E")
    ax.set_title("Maximum number densities vs. ionospheric layer")
    ax.set_ylabel("(m$^{-3}$)")
    ax.set_yscale("log")
    ax.legend(loc="best")
    ax = axs[1]
    ax.plot(iono.time, iono["hmF2"], label="h$_m$F$_2$")
    ax.plot(iono.time, iono["hmF1"], label="h$_m$F$_1$")
    ax.plot(iono.time, iono["hmE"], label="h$_m$E")
    ax.set_title("Height of maximum density vs. ionospheric layer")
    ax.set_ylabel("(km)")
    ax.set_ylim((90, None))
    ax.legend(loc="best")
    ax = axs[2]
    ax.plot(iono.time, iono["foF2"], label="foF2")
    ax.set_title("F2 layer plasma frequency")
    ax.set_ylabel("(MHz)")

    for a in axs.ravel():
        a.grid(True)

    # %%
    fig = figure(figsize=(16, 12))
    axs = fig.subplots(1, 1, sharex=True)

    fig.suptitle(
        f"{str(iono.time[0].values)[:-13]} to "
        f"{str(iono.time[-1].values)[:-13]}\n"
        f"Glat, Glon: {iono.glat.item()}, {iono.glon.item()}"
    )
    # %% Tec(time)
    ax = axs
    ax.plot(iono.time, iono["TEC"], label="TEC")
    ax.set_ylabel("(m$^{-2}$)")
    ax.set_title("Total Electron Content (TEC)")
    # ax.set_yscale('log')
    ax.legend(loc="best")
    ax.grid(True)
    # %% ion_drift(time)
    # ax = axs[1]
    # ax.plot(iono.time, iono["EqVertIonDrift"], label=r"V$_y$")
    # ax.set_xlabel("time (UTC)")
    # ax.set_ylabel("(m/s)")
    # ax.legend(loc="best")

    # for a in axs.ravel():
    #    a.grid(True)

    # %%  Ne(time)
    fg = figure()
    ax = fg.gca()
    hi = ax.pcolormesh(iono.time, iono.alt_km, iono["ne"].values.T)
    fg.colorbar(hi, ax=ax).set_label("[m$^{-3}$]")
    ax.set_ylabel("altitude [km]")
    ax.set_title(f"$N_e$ vs. altitude and time")


def altprofile(iono: xarray.Dataset):
    fig = figure(figsize=(16, 6))
    axs = fig.subplots(1, 2)

    fig.suptitle(f"{str(iono.time[0].values)[:-13]}\n" f"Glat, Glon: {iono.glat.item()}, {iono.glon.item()}")

    pn = axs[0]
    pn.plot(iono["ne"], iono.alt_km, label="N$_e$")
    # pn.set_title(iri2016Obj.title1)
    pn.set_xlabel("Density (m$^{-3}$)")
    pn.set_ylabel("Altitude (km)")
    pn.set_xscale("log")
    pn.legend(loc="best")
    pn.grid(True)

    pn = axs[1]
    pn.plot(iono["Ti"], iono.alt_km, label="T$_i$")
    pn.plot(iono["Te"], iono.alt_km, label="T$_e$")
    # pn.set_title(iri2016Obj.title2)
    pn.set_xlabel("Temperature (K)")
    pn.set_ylabel("Altitude (km)")
    pn.legend(loc="best")
    pn.grid(True)


def latprofile(iono: xarray.Dataset):

    fig = figure(figsize=(8, 12))
    axs = fig.subplots(2, 1, sharex=True)

    ax = axs[0]

    ax.plot(iono["glat"], iono["NmF2"], label="N$_m$F$_2$")
    ax.plot(iono["glat"], iono["NmF1"], label="N$_m$F$_1$")
    ax.plot(iono["glat"], iono["NmE"], label="N$_m$E")
    ax.set_title(str(iono.time[0].values)[:-13] + f'  latitude {iono["glat"][[0, -1]].values}')
    # ax.set_xlim(iono.lat[[0, -1]])
    ax.set_xlabel(r"Geog. Lat. ($^\circ$)")
    ax.set_ylabel("(m$^{-3}$)")
    ax.set_yscale("log")

    ax = axs[1]
    ax.plot(iono["glat"], iono["hmF2"], label="h$_m$F$_2$")
    ax.plot(iono["glat"], iono["hmF1"], label="h$_m$F$_1$")
    ax.plot(iono["glat"], iono["hmE"], label="h$_m$E")
    ax.set_xlim(iono["glat"][[0, -1]])
    ax.set_title(str(iono.time[0].values)[:-13] + f'  latitude  {iono["glat"][[0, -1]].values}')
    ax.set_xlabel(r"Geog. Lat. ($^\circ$)")
    ax.set_ylabel("(km)")

    for a in axs:
        a.legend(loc="best")
        a.grid(True)
